==> truffle-api-21.1.0-sources.jar: com/oracle/truffle/api/dsl/AOTSupport.java <==
/*
 * Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.api.dsl;

import com.oracle.truffle.api.TruffleLanguage;
import com.oracle.truffle.api.nodes.Node;
import com.oracle.truffle.api.nodes.NodeUtil;
import com.oracle.truffle.api.nodes.NodeVisitor;
import com.oracle.truffle.api.nodes.RootNode;

/**
 * Helper class to prepare AOT enabled nodes for AOT in {@link RootNode#prepareForAOT}.
 * <p>
 * See also the <a href= "https://github.com/oracle/graal/blob/master/truffle/docs/AOT.md">usage
 * tutorial</a> on the website.
 *
 * @see GenerateAOT
 * @since 21.1
 */
public final class AOTSupport {

    /*
     * No instances.
     */
    private AOTSupport() {
    }

    /**
     * Recursively prepares an AST for AOT compilation. If an AST is already prepared for AOT then
     * this method has no effect. The AST lock will be held during this operation.
     *
     * @see GenerateAOT
     * @see RootNode#prepareForAOT
     * @since 21.1
     */
    public static void prepareForAOT(RootNode root) {
        root.atomic(new Runnable() {
            public void run() {
                final TruffleLanguage<?> language = DSLAccessor.nodeAccessor().getLanguage(root);
                NodeUtil.forEachChild(root, new NodeVisitor() {
                    public boolean visit(Node node) {
                        if (node instanceof GenerateAOT.Provider) {
                            GenerateAOT.Provider provider = (GenerateAOT.Provider) node;
                            provider.prepareForAOT(language, node.getRootNode());
                        }
                        return NodeUtil.forEachChild(node, this);
                    }
                });
            }
        });
    }

}

==> truffle-api-21.3.3.1-sources.jar: com/oracle/truffle/api/dsl/AOTSupport.java <==
/*
 * Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.api.dsl;

import com.oracle.truffle.api.TruffleLanguage;
import com.oracle.truffle.api.nodes.Node;
import com.oracle.truffle.api.nodes.NodeUtil;
import com.oracle.truffle.api.nodes.NodeVisitor;
import com.oracle.truffle.api.nodes.RootNode;

/**
 * Helper class to prepare AOT enabled nodes for AOT in {@link RootNode#prepareForAOT}.
 * <p>
 * See also the <a href= "https://github.com/oracle/graal/blob/master/truffle/docs/AOT.md">usage
 * tutorial</a> on the website.
 *
 * @see GenerateAOT
 * @since 21.1
 */
public final class AOTSupport {

    /*
     * No instances.
     */
    private AOTSupport() {
    }

    /**
     * Recursively prepares an AST for AOT compilation. If an AST is already prepared for AOT then
     * this method has no effect. The AST lock will be held during this operation.
     *
     * @see GenerateAOT
     * @see RootNode#prepareForAOT
     * @since 21.1
     */
    public static void prepareForAOT(RootNode root) {
        root.atomic(new Runnable() {
            public void run() {
                final TruffleLanguage<?> language = DSLAccessor.nodeAccessor().getLanguage(root);
                NodeUtil.forEachChild(root, new NodeVisitor() {
                    public boolean visit(Node node) {
                        if (node instanceof GenerateAOT.Provider) {
                            GenerateAOT.Provider provider = (GenerateAOT.Provider) node;
                            if (node.isAdoptable() && node.getRootNode() == null) {
                                throw new AssertionError("Node is not yet adopted before prepare " + node);
                            }
                            provider.prepareForAOT(language, root);
                        }
                        return NodeUtil.forEachChild(node, this);
                    }
                });
            }
        });
    }

}
